# -*- coding: utf-8 -*-
"""
Import this module to add the validation tween to your pyramid app.
"""
import pyramid
from .api import register_swagger_endpoints


def includeme(config):
    config.add_tween(
        "pyramid_swagger.tween.validation_tween_factory",
        under=pyramid.tweens.EXCVIEW
    )
    register_swagger_endpoints(config)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
from contextlib import contextmanager
from pyramid.interfaces import IRoutesMapper

from collections import namedtuple
import re

import jsonschema.exceptions
import simplejson
from jsonschema.validators import Draft3Validator, Draft4Validator
from pyramid_swagger.exceptions import RequestValidationError
from pyramid_swagger.exceptions import ResponseValidationError
from .ingest import compile_swagger_schema
from .model import PathNotMatchedError


EXTENDED_TYPES = {
    'float': (float,),
    'int': (int,),
}

DEFAULT_EXCLUDED_PATHS = [
    r'^/static/?',
    r'^/api-docs/?'
]


class Settings(namedtuple(
    'Settings',
    [
        'schema_dir',
        'validate_swagger_spec',
        'validate_request',
        'validate_response',
        'validate_path',
        'exclude_paths',
    ]
)):

    """A settings object for configuratble options.

    :param schema_dir: location of Swagger schema files.
    :param validate_swagger_spec: check Swagger files for correctness.
    :param validate_request: check requests against Swagger spec.
    :param validate_response: check responses against Swagger spec.
    :param validate_path: check if request path is in schema. If disabled
        and path not found in schema, request / response validation is skipped.
    :param exclude_paths: list of paths (in regex format) that should be
        excluded from validation.
    :rtype: namedtuple
    """


@contextmanager
def noop_context(request, response=None):
    yield


def _get_validation_context(registry):
    validation_context_path = registry.settings.get(
        'pyramid_swagger.validation_context_path',
    )

    if validation_context_path:
        m = re.match(
            '(?P<module_path>.*)\.(?P<contextmanager_name>.*)',
            validation_context_path,
        )
        module_path = m.group('module_path')
        contextmanager_name = m.group('contextmanager_name')

        return getattr(
            __import__(module_path, fromlist=contextmanager_name),
            contextmanager_name,
        )
    else:
        return noop_context


def validation_tween_factory(handler, registry):
    """Pyramid tween for performing validation.

    Note this is very simple -- it validates requests, responses, and paths
    while delegating to the relevant matching view.
    """
    settings = load_settings(registry)
    schema = compile_swagger_schema(
        settings.schema_dir,
        settings.validate_swagger_spec
    )
    route_mapper = registry.queryUtility(IRoutesMapper)
    disable_all_validation = not any((
        settings.validate_request,
        settings.validate_response,
        settings.validate_path
    ))

    def validator_tween(request):
        if disable_all_validation or \
                should_exclude_path(settings.exclude_paths, request.path):
            return handler(request)

        validation_context = _get_validation_context(registry)

        try:
            schema_data, resolver = schema.schema_and_resolver_for_request(
                request)
        except PathNotMatchedError as exc:
            if settings.validate_path:
                with validation_context(request):
                    raise RequestValidationError(str(exc))
            else:
                return handler(request)

        if settings.validate_request:
            with validation_context(request):
                _validate_request(
                    route_mapper,
                    request,
                    schema_data,
                    resolver
                )

        response = handler(request)

        if settings.validate_response:
            with validation_context(request, response=response):
                _validate_response(
                    response,
                    schema_data,
                    resolver
                )

        return response

    return validator_tween


def load_settings(registry):
    return Settings(
        # By default, assume cwd contains the swagger schemas.
        schema_dir=registry.settings.get(
            'pyramid_swagger.schema_directory',
            'api_docs/'
        ),
        validate_swagger_spec=registry.settings.get(
            'pyramid_swagger.enable_swagger_spec_validation',
            True
        ),
        validate_request=registry.settings.get(
            'pyramid_swagger.enable_request_validation',
            True
        ),
        validate_response=registry.settings.get(
            'pyramid_swagger.enable_response_validation',
            True
        ),
        validate_path=registry.settings.get(
            'pyramid_swagger.enable_path_validation',
            True
        ),
        exclude_paths=get_exclude_paths(registry),
    )


def get_exclude_paths(registry):
    """Compiles a list of paths that should not be validated against.
        :rtype: list of compiled validation regexes
    """
    # TODO(#63): remove deprecated `skip_validation` setting in v2.0.
    regexes = registry.settings.get(
        'pyramid_swagger.skip_validation',
        registry.settings.get(
            'pyramid_swagger.exclude_paths',
            DEFAULT_EXCLUDED_PATHS
        )
    )

    # being nice to users using strings :p
    if not isinstance(regexes, list) and not isinstance(regexes, tuple):
        regexes = [regexes]

    return [re.compile(r) for r in regexes]


def should_exclude_path(exclude_path_regexes, path):
    # Skip validation for the specified endpoints
    return any(r.match(path) for r in exclude_path_regexes)


def _validate_request(route_mapper, request, schema_data, resolver):
    """ Validates a request and raises a RequestValidationError on failure.

    :param request: the request object to validate
    :type request: Pyramid request object passed into a view
    :param schema_data: our mapping from request data to schemas (see
        load_schema)
    :type schema_data: dict
    :param resolver: the request object to validate
    :type resolver: Pyramid request object passed into a view
    """
    try:
        validate_incoming_request(
            route_mapper,
            request,
            schema_data,
            resolver
        )
    except jsonschema.exceptions.ValidationError as exc:
        # This will alter our stack trace slightly, but Pyramid knows how
        # to render it. And the real value is in the message anyway.
        raise RequestValidationError(str(exc))


def _validate_response(response, schema_data, schema_resolver):
    """ Validates a response and raises a ResponseValidationError on failure.

    :param response: the response object to validate
    :type response: Pyramid response object passed into a view
    :param schema_data: our mapping from request data to schemas (see
        load_schema)
    :type schema_data: dict
    :param resolver: the request object to validate
    :type resolver: Pyramid request object passed into a view
    """
    try:
        validate_outgoing_response(
            response,
            schema_data,
            schema_resolver
        )
    except jsonschema.exceptions.ValidationError as exc:
        # This will alter our stack trace slightly, but Pyramid knows how
        # to render it and the real value is in the message anyway.
        raise ResponseValidationError(str(exc))


def cast_request_param(request_schema, param_name, param_value):
    """Try to cast a request param (e.g. query arg, POST data) from a string to
    its specified type in the schema. This allows validating non-string params.

    :param request_schema: request schema
    :type request_schema: dict
    :param param_name: param name
    :type: string
    :param param_name: param value
    :type: string
    """
    type_to_cast_fn = {
        'integer': int,
        'float': float,
        'boolean': bool,
    }

    param_type = request_schema['properties'].get(param_name, {}).get('type')
    try:
        return type_to_cast_fn.get(param_type, lambda x: x)(param_value)
    except ValueError:
        # Ignore type error, let jsonschema validation handle incorrect types
        return param_value


def validate_incoming_request(route_mapper, request, schema_map, resolver):
    """Validates an incoming request against our schemas.

    :param request: the request object to validate
    :type request: Pyramid request object passed into a view
    :param schema_map: our mapping from request data to schemas (see
        load_schema)
    :type schema_map: dict
    :param resolver: the request object to validate
    :type resolver: Pyramid request object passed into a view
    :returns: None
    """
    if schema_map.request_query_schema:
        # You'll notice we use Draft3 some places and Draft4 in others.
        # Unfortunately this is just Swagger's inconsistency showing. It
        # may be nice in the future to do the necessary munging to make
        # everything Draft4 compatible, although the Swagger UI will
        # probably never truly support Draft4.
        request_query_params = dict(
            (k, cast_request_param(schema_map.request_query_schema, k, v))
            for k, v
            in request.GET.items()
        )
        Draft3Validator(
            schema_map.request_query_schema,
            resolver=resolver,
            types=EXTENDED_TYPES,
        ).validate(request_query_params)

    if schema_map.request_path_schema:
        # We don't have access to this yet but let's go ahead and build the
        # matchdict so we can validate it.
        info = route_mapper(request)
        matchdict = dict(
            (k, cast_request_param(schema_map.request_path_schema, k, v))
            for k, v
            in info.get('match', {}).items()
        )
        Draft3Validator(
            schema_map.request_path_schema,
            resolver=resolver,
            types=EXTENDED_TYPES,
        ).validate(matchdict)

    # Body validation
    if schema_map.request_body_schema:
        body = getattr(request, 'json_body', {})
        Draft4Validator(
            schema_map.request_body_schema,
            resolver=resolver,
            types=EXTENDED_TYPES,
        ).validate(body)


def validate_outgoing_response(response, schema_map, resolver):
    """Validates response against our schemas.

    :param response: the response object to validate
    :type response: Requests response object
    :param schema_map: our mapping from request data to schemas (see
        load_schema)
    :type schema_map: dict
    :param resolver: a resolver for validation, if any
    :type resolver: a jsonschema resolver or None
    :returns: None
    """
    body = prepare_body(response)
    # Short circuit if we are supposed to not validate anything.
    if schema_map.response_body_schema.get('type') == 'void' and body is None:
        return
    Draft4Validator(
        schema_map.response_body_schema,
        resolver=resolver,
        types=EXTENDED_TYPES,
    ).validate(body)


def prepare_body(response):
    # content_type and charset must both be set to access response.text
    if response.content_type is None or response.charset is None:
        raise ResponseValidationError(
            'Response validation error: Content-Type and charset must be set'
        )

    if 'application/json' in response.content_type:
        return simplejson.loads(response.text)
    else:
        return response.text



# -*- coding: utf-8 -*-
"""
Methods to help validate a given JSON document against the Swagger Spec.
"""
from pkg_resources import resource_filename

import jsonschema
import simplejson
from jsonschema.validators import RefResolver


def validate_swagger_schemas(resource_listing, resources):
    """Validate the structure of Swagger schemas against the spec.

    :param resource_listing: A filepath to a Swagger resource listing
    :type resource_listing: string
    :param resources: A list of filespaths to Swagger API declarations
    :type resources: [string]
    :raises: jsonschema ValidationErrors on malformed schemas
    """
    with open(resource_listing) as listing_file:
        validate_resource_listing(simplejson.load(listing_file))

    for resource in resources:
        with open(resource) as resource_file:
            resource_json = simplejson.load(resource_file)
        validate_api_declaration(resource_json)


def validate_resource_listing(resource_listing_json):
    resource_spec_path = resource_filename(
        'pyramid_swagger',
        'swagger_spec_schemas/v1.2/resourceListing.json'
    )
    validate_jsonschema(resource_spec_path, resource_listing_json)


def validate_api_declaration(api_declaration_json):
    """Validate a swagger schema.

    :param schema_dir: the directory schema files live inside
    :type schema_dir: string
    """
    api_spec_path = resource_filename(
        'pyramid_swagger',
        'swagger_spec_schemas/v1.2/apiDeclaration.json'
    )
    validate_jsonschema(api_spec_path, api_declaration_json)


def validate_jsonschema(spec_path, json_object):
    with open(spec_path) as schema_file:
        schema = simplejson.loads(schema_file.read())
        resolver = RefResolver(
            "file://{0}".format(spec_path),
            schema
        )
        jsonschema.validate(json_object, schema, resolver=resolver)



# -*- coding: utf-8 -*-
"""
Module for automatically serving /api-docs* via Pyramid.
"""
import simplejson

from .ingest import compile_swagger_schema
from .tween import load_settings


def register_swagger_endpoints(config):
    """Create and register pyramid endpoints for /api-docs*."""
    settings = load_settings(config.registry)
    swagger_schema = compile_swagger_schema(
        settings.schema_dir,
        settings.validate_swagger_spec,
    )
    with open(swagger_schema.resource_listing) as input_file:
        register_resource_listing(config, simplejson.load(input_file))

    for name, filepath in swagger_schema.api_declarations.items():
        with open(filepath) as input_file:
            register_api_declaration(
                config,
                name,
                simplejson.load(input_file)
            )


def register_resource_listing(config, resource_listing):
    """Registers the endpoint /api-docs.

    :param config: Configurator instance for our webapp
    :type config: pyramid Configurator
    :param resource_listing: JSON representing a resource listing
    :type resource_listing: dict
    """
    def view_for_resource_listing(request):
        # Thanks to the magic of closures, this means we gracefully return JSON
        # without file IO at request time.
        return resource_listing

    route_name = 'api_docs'
    config.add_route(route_name, '/api-docs')
    config.add_view(
        view_for_resource_listing,
        route_name=route_name,
        renderer='json'
    )


def register_api_declaration(config, resource_name, api_declaration):
    """Registers an endpoint at /api-docs.

    :param config: Configurator instance for our webapp
    :type config: pyramid Configurator
    :param resource_name: The `path` parameter from the resource listing for
        this resource.
    :type resource_name: string
    :param api_declaration: JSON representing an api declaration
    :type api_declaration: dict
    """
    # NOTE: This means our resource paths are currently constrained to be valid
    # pyramid routes! (minus the leading /)
    route_name = 'apidocs-{0}'.format(resource_name)
    config.add_route(route_name, '/api-docs/{0}'.format(resource_name))
    config.add_view(
        build_api_declaration_view(api_declaration),
        route_name=route_name,
        renderer='json'
    )


def build_api_declaration_view(api_declaration_json):
    """Thanks to the magic of closures, this means we gracefully return JSON
    without file IO at request time."""
    def view_for_api_declaration(request):
        # Note that we rewrite basePath to always point at this server's root.
        return dict(
            api_declaration_json,
            basePath=str(request.application_url),
        )
    return view_for_api_declaration



# -*- coding: utf-8 -*-
"""
The core model we use to represent the entire ingested swagger schema for this
service.
"""
import re


class PathNotMatchedError(Exception):
    """Raised when a SwaggerSchema object is given a request it cannot match
    against its stored schema."""
    pass


class SwaggerSchema(object):
    """
    This object contains data structures representing your Swagger schema
    and exposes methods for efficiently finding the relevant schemas for a
    Pyramid request.
    """

    def __init__(self, resource_listing, api_declarations, schema_resolvers):
        """Store schema_resolvers for later use.

        :param resource_listing: Filepath to a resource listing
        :type resource_listing: string
        :param api_declarations: Map from resource name to filepath of its api
            declaration
        :type api_declarations: dict
        :param schema_resolvers: a list of resolvers, one per Swagger resource
        :type schema_resolvers: list of
            pyramid_swagger.load_schema.SchemaAndResolver objects
        """
        self.resource_listing = resource_listing
        self.api_declarations = api_declarations
        self.schema_resolvers = schema_resolvers

    def schema_and_resolver_for_request(self, request):
        """Takes a request and returns the relevant schema, ready for
        validation.

        :param request: A Pyramid request to fetch schemas for
        :type request: pyramid.request.Request
        :returns: (schema_map, resolver) for this particular request
        :rtype: A tuple of (load_schema.SchemaMap, jsonschema.Resolver)
        """
        for schema_resolver in self.schema_resolvers:
            request_to_schema_map = schema_resolver.request_to_schema_map
            resolver = schema_resolver.resolver
            for (path, method), schema_map in request_to_schema_map.items():
                if partial_path_match(request.path, path) \
                        and method == request.method:
                    return (schema_map, resolver)

        raise PathNotMatchedError(
            'Could not find the relevant path ({0}) in the Swagger schema. '
            'Perhaps you forgot to add it?'.format(request.path)
        )


def partial_path_match(path1, path2, kwarg_re=r'\{.*\}'):
    """Validates if path1 and path2 matches, ignoring any kwargs in the string.

    We need this to ensure we can match Swagger patterns like:
        /foo/{id}
    against the observed pyramid path
        /foo/1

    :param path1: path of a url
    :type path1: string
    :param path2: path of a url
    :type path2: string
    :param kwarg_re: regex pattern to identify kwargs
    :type kwarg_re: regex string
    :returns: boolean
    """
    split_p1 = path1.split('/')
    split_p2 = path2.split('/')
    pat = re.compile(kwarg_re)
    if len(split_p1) != len(split_p2):
        return False
    for partial_p1, partial_p2 in zip(split_p1, split_p2):
        if pat.match(partial_p1) or pat.match(partial_p2):
            continue
        if not partial_p1 == partial_p2:
            return False
    return True



# -*- coding: utf-8 -*-
"""
Module to load swagger specs and build efficient data structures for querying
them during request validation.
"""
from __future__ import unicode_literals
from collections import namedtuple

import simplejson
from jsonschema import RefResolver


def extract_query_param_schema(schema):
    """Turn a swagger endpoint schema into an equivalent one to validate our
    request.

    As an example, this would take this swagger schema:
        {
            "paramType": "query",
            "name": "query",
            "description": "Location to query",
            "type": "string",
            "required": true
        }
    To this jsonschema:
        {
            "type": "object",
            "additionalProperties": "False",
            "properties:": {
                "description": "Location to query",
                "type": "string",
                "required": true
            }
        }
    Which we can then validate against a JSON object we construct from the
    pyramid request.
    """
    properties = dict(
        (s['name'], strip_swagger_markup(s))
        for s in schema['parameters']
        if s['paramType'] == 'query'
    )
    # Generate a jsonschema that describes the set of all query parameters. We
    # can then validate this against dict(request.params).
    if properties:
        return {
            'type': 'object',
            'properties': properties,
            'additionalProperties': False,
        }
    else:
        return None


def extract_path_schema(schema):
    """Extract a schema for path variables for an endpoint.

    As an example, this would take this swagger schema:
        {
            "paramType": "path",
            "type": "string",
            "enum": ["foo", "bar"],
            "required": true
        }
    To this jsonschema:
        {
            "type": "string",
            "enum": ["foo", "bar"],
        }
    Which we can then validate against a JSON object we construct from the
    pyramid request.
    """
    properties = dict(
        (s['name'], strip_swagger_markup(s))
        for s in schema['parameters']
        if s['paramType'] == 'path'
    )
    if properties:
        return {
            'type': 'object',
            'properties': properties,
            'additionalProperties': False,
        }
    else:
        return None


def extract_body_schema(schema, models_schema):
    """Turn a swagger endpoint schema into an equivalent one to validate our
    request.

    As an example, this would take this swagger schema:
        {
            "paramType": "body",
            "name": "body",
            "description": "json list: [ll1,ll2]",
            "type": "array",
            "items": {
                "$ref": "GeoPoint"
            },
            "required": true
        }
    To this jsonschema:
        {
            "type": "array",
            "items": {
                "$ref": "GeoPoint"
            },
        }
    Which we can then validate against a JSON object we construct from the
    pyramid request.
    """
    matching_body_schemas = [
        s
        for s in schema['parameters']
        if s['paramType'] == 'body'
    ]
    if matching_body_schemas:
        schema = matching_body_schemas[0]
        type_ref = extract_validatable_type(schema['type'], models_schema)
        # Unpleasant, but we are forced to replace 'type' defns with proper
        # jsonschema refs.
        if '$ref' in type_ref:
            del schema['type']
            schema.update(type_ref)
        return strip_swagger_markup(schema)
    else:
        return None


def strip_swagger_markup(schema):
    """Turn a swagger URL parameter schema into a raw jsonschema.

    Involves just removing various swagger-specific markup tags.
    """
    swagger_specific_keys = (
        'paramType',
        'name',
    )
    return dict(
        (k, v)
        for k, v in schema.items()
        if k not in swagger_specific_keys
    )


def get_model_resolver(schema):
    """
    Gets the schema and a RefResolver. RefResolver's will resolve "$ref:
    ObjectType" entries in the schema, which are used to describe more complex
    objects.

    :returns: The RefResolver for the schema's models.
    :rtype: RefResolver
    """
    models = dict(
        (k, strip_swagger_markup(v))
        for k, v in schema.get('models', {}).items()
    )
    return RefResolver('', '', models)


class SchemaMap(namedtuple(
        'SchemaMap', [
            'request_query_schema',
            'request_path_schema',
            'request_body_schema',
            'response_body_schema'
        ])):
    """
    A SchemaMap contains a mapping from incoming paths to schemas for request
    queries, request bodies, and responses. This requires some precomputation
    but means we can do fast query-time validation without having to walk over
    the schema.
    """
    __slots__ = ()


def build_request_to_schemas_map(schema):
    """Take the swagger schema and build a map from incoming path to a
    jsonschema for requests and responses."""
    request_to_schema = {}
    schema_models = schema.get('models', {})
    for api in schema['apis']:
        path = api['path']
        for operation in api['operations']:
            # Now that we have the necessary info for this particular
            # path/method combination, build our dict.
            key = (path, operation['method'])
            request_to_schema[key] = SchemaMap(
                request_query_schema=extract_query_param_schema(operation),
                request_path_schema=extract_path_schema(operation),
                request_body_schema=extract_body_schema(
                    operation,
                    schema_models
                ),
                response_body_schema=extract_response_body_schema(
                    operation,
                    schema_models
                ),
            )

    return request_to_schema


def extract_response_body_schema(operation, schema_models):
    if operation['type'] in schema_models:
        return extract_validatable_type(operation['type'], schema_models)
    else:
        acceptable_fields = (
            'type', '$ref', 'format', 'defaultValue', 'enum', 'minimum',
            'maximum', 'items', 'uniqueItems'
        )

        schema = dict([
            (field, operation[field])
            for field in acceptable_fields
            if field in operation
        ])

        return schema


def extract_validatable_type(type_name, models):
    """Returns a jsonschema-compatible typename from the Swagger type.

    This is necessary because for our Swagger specs to be compatible with
    swagger-ui, they must not use a $ref to internal models.

    :returns: A key-value that jsonschema can validate. Key will be either
        'type' or '$ref' as is approriate.
    :rtype: dict
    """
    if type_name in models:
        return {'$ref': type_name}
    else:
        return {'type': type_name}


class SchemaAndResolver(namedtuple(
        'SAR',
        ['request_to_schema_map', 'resolver'])):
    __slots__ = ()


def load_schema(schema_path):
    """Prepare the schema so we can make fast validation comparisons.

    The prepared schema will be a map:
        key: (swagger_path, method) e.g. ('/v1/reverse', 'GET')
        value: a SchemaMap

    For any request, you just need to:
        1) Validate {k, v for k, v in query.params} against
            request_query_schema
        2) Validate request body against request_body_schema
        3) Validate response body against response_body_schema

        Response and request bodies will need to be transformed as indicated by
        their content type (e.g. simplejson.loads if you have application/json
        type).

    :returns: SchemaAndResolver
    """
    with open(schema_path, 'r') as schema_file:
        schema = simplejson.load(schema_file)
    return SchemaAndResolver(
        request_to_schema_map=build_request_to_schemas_map(schema),
        resolver=get_model_resolver(schema),
    )



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import os.path
import simplejson

from .load_schema import load_schema
from .model import SwaggerSchema
from .spec import validate_swagger_schemas


API_DOCS_FILENAME = 'api_docs.json'


class ResourceListingNotFoundError(Exception):
    pass


class ApiDeclarationNotFoundError(Exception):
    pass


def find_resource_names(api_docs_json):
    return [
        api['path'].lstrip('/')
        for api in api_docs_json['apis']
    ]


def build_schema_mapping(schema_dir):
    """Discovers schema file locations and relations.

    :param schema_dir: the directory schema files live inside
    :type schema_dir: string
    :returns: A tuple of (resource listing filepath, mapping) where the mapping
        is between resource name and file path
    :rtype: (string, dict)
    """
    def resource_name_to_filepath(name):
        return os.path.join(schema_dir, '{0}.json'.format(name))

    listing, listing_json = _load_resource_listing(schema_dir)

    return (
        listing,
        dict(
            (resource, resource_name_to_filepath(resource))
            for resource in find_resource_names(listing_json)
        )
    )


def _load_resource_listing(schema_dir):
    """Load the resource listing from file, handling errors.

    :param schema_dir: the directory schema files live inside
    :type schema_dir: string
    :returns: (resource listing filepath, resource listing json)
    """
    resource_listing = os.path.join(schema_dir, API_DOCS_FILENAME)
    try:
        with open(resource_listing) as resource_listing_file:
            resource_listing_json = simplejson.load(resource_listing_file)
    # If not found, raise a more user-friendly error.
    except IOError:
        raise ResourceListingNotFoundError(
            'No resource listing found at {0}. Note that your json file '
            'must be named {1}'.format(resource_listing, API_DOCS_FILENAME)
        )
    return resource_listing, resource_listing_json


def compile_swagger_schema(schema_dir, should_validate_schemas):
    """Build a SwaggerSchema from various files.

    :param schema_dir: the directory schema files live inside
    :type schema_dir: string
    :param should_validate_schemas: if True, check schemas for correctness
    :type should_validate_schemas: boolean
    :returns: a SwaggerSchema object
    """
    listing, mapping = build_schema_mapping(schema_dir)
    schema_resolvers = ingest_resources(
        listing,
        mapping,
        schema_dir,
        should_validate_schemas,
    )
    return SwaggerSchema(
        listing,
        mapping,
        schema_resolvers,
    )


def ingest_resources(listing, mapping, schema_dir, should_validate_schemas):
    """Consume the Swagger schemas and produce a queryable datastructure.

    :param listing: Filepath to a resource listing
    :type listing: string
    :param mapping: Map from resource name to filepath of its api declaration
    :type mapping: dict
    :param schema_dir: the directory schema files live inside
    :type schema_dir: string
    :param should_validate_schemas: if True, check schemas for correctness
    :type should_validate_schemas: boolean
    :returns: A list of SchemaAndResolver objects
    """
    resource_filepaths = mapping.values()

    ingested_resources = []
    for name, filepath in mapping.items():
        try:
            ingested_resources.append(load_schema(filepath))
        # If we have trouble reading any files, raise a more user-friendly
        # error.
        except IOError:
            raise ApiDeclarationNotFoundError(
                'No api declaration found at {0}. Attempted to load the `{1}` '
                'resource relative to the schema_directory `{2}`. Perhaps '
                'your resource name and API declaration file do not '
                'match?'.format(filepath, name, schema_dir)
            )

    if should_validate_schemas:
        validate_swagger_schemas(
            listing,
            resource_filepaths
        )

    return ingested_resources



# -*- coding: utf-8 -*-
from pyramid.httpexceptions import HTTPClientError, HTTPInternalServerError


class RequestValidationError(HTTPClientError):
    pass


class ResponseValidationError(HTTPInternalServerError):
    pass
